/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   Contains parts of code from klcc project                              *
*   (Peter Simonsson <psn@linux.se>.                                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <time.h>
#include <qstringlist.h>
#include <qcstring.h>
#include <qptrlist.h>
#include <qcheckbox.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qtimer.h>
#include <qspinbox.h>
#include <qstringlist.h>
#include <qcombobox.h>
#include <qfile.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <kstdaction.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpassdlg.h>
#include <knotifyclient.h>
#include <kstatusbar.h>
#include <kdebug.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <knotifydialog.h>
#include <kapplication.h>
#include <klineedit.h>
#include <klocale.h>
#include <kpassdlg.h>
#include <kurlrequester.h>
#include <kcombobox.h>
#include <kstddirs.h>
#include <klineeditdlg.h>
#include <kmessagebox.h>
#include "kvpnc.h"
#include "mainview.h"
#include "mainviewbase.h"
#include "preferencesdialog.h"
#include "helpdialog.h"
#include "advancedsettingsdialog.h"

KVpnc::KVpnc( QWidget*, const char *name ) : KMainWindow( 0, name )
{

	config = kapp->config();
	setCaption( i18n( "KVpnc" ) );

	disconnectedIcon = KGlobal::iconLoader() ->loadIcon( "kvpnc-disconnected", KIcon::NoGroup, 22 );
	connectedIcon = KGlobal::iconLoader() ->loadIcon( "kvpnc-connected", KIcon::NoGroup, 22 );

	setIcon( disconnectedIcon );

	writeLogFile = true;
	KStandardDirs * dirs = KGlobal::dirs();
	logfileName = dirs->saveLocation( "data" );
	logfileName += "/kvpnc/kvpnc.log";
	logfile.setName( logfileName );
	s.setDevice( &logfile );

	this->initStatusbar();
	this->initAction();
	this->initDockWindow();
	this->initView();
	this->loadOptions();


	isConnected = false;
	mw->buttonConnect->setEnabled( true );
	mw->buttonDisconnect->setEnabled( false );

	minimizeAfterConnect = true;
	//showDebugConsole = false;
	QString pathToVpnc = "";
	QString pathToVpncConnect = "";
	QString pathToVpncDisconnect = "";
	connectCounter = 0;
	VpncDebugLevel = 0;
	connectSleepDelay = 15;
	connectSleepRetry = 0;
	tryConnectTimeout = 10;
	runningCheckCounter = 0;
	runningCheckTimeout = 60;
	lastProfile = "";
	AccountList.setAutoDelete( TRUE ); // the list owns the objects

}

KVpnc::~KVpnc()
{}

void KVpnc::initAction()
{
	( void* ) KStdAction::quit( this, SLOT( quitCalled() ), actionCollection() );

	saveSessionAction = new KAction( i18n( "&Save profile..." ), "fileexport", Key_S, this, SLOT( saveSessionClicked() ), actionCollection(), "saveSession" );
	deleteSessionAction = new KAction( i18n( "&Delete profile..." ), "editdelete", Key_D, this, SLOT( deleteSessionClicked() ), actionCollection(), "deleteSession" );
	newSessionAction = new KAction( i18n( "&New profile..." ), "newfont", Key_N, this, SLOT( newSessionClicked() ), actionCollection(), "newSession" );
	advancedSessionAction = new KAction( i18n( "&Advanced..." ), "wizard", Key_A, this, SLOT( advancedSettingsClicked() ), actionCollection(), "advancedSettingsSession" );



	connectAction = new KAction( i18n( "&Connect" ), "up", CTRL + Key_Up, this, SLOT( connectClicked() ), actionCollection(), "serverConnect" );
	disconnectAction = new KAction( i18n( "&Disconnect" ), "down", CTRL + Key_Down, this, SLOT( disconnectClicked() ), actionCollection(), "serverDisconnect" );

	this->connectAction->setEnabled( true );
	this->disconnectAction->setEnabled( false );

                                                this->deleteSessionAction->setEnabled ( FALSE );
                                                this->advancedSessionAction->setEnabled ( FALSE );
                                                this->saveSessionAction->setEnabled ( FALSE );


	showToolbarAction = KStdAction::showToolbar( this, SLOT( slotShowToolbar() ), actionCollection() );
	showStatusbarAction = KStdAction::showStatusbar( this, SLOT( slotShowStatusbar() ), actionCollection() );
	preferencesAction = KStdAction::preferences( this, SLOT( slotSettings() ), actionCollection() );

	( void* ) KStdAction::keyBindings( this, SLOT( slotKeyBindings() ), actionCollection() );
	( void* ) KStdAction::configureToolbars( this, SLOT( slotConfigToolbars() ), actionCollection() );
	( void* ) KStdAction::configureNotifications( this, SLOT( slotConfigNotifications() ), actionCollection() );

	createGUI();
}

void KVpnc::initView()
{

	mw = new MainView( this, "mainView" );
	setCentralWidget( mw );

	connect ( mw->buttonHelp, SIGNAL( clicked() ), this, SLOT( helpClicked() ) );
	connect ( mw->buttonConnect, SIGNAL( clicked() ), this, SLOT( connectClicked() ) );
	connect ( mw->buttonDisconnect, SIGNAL( clicked() ), this, SLOT( disconnectClicked() ) );

	connect ( mw->SessionCombo, SIGNAL ( activated ( const QString & ) ), this, SLOT ( sessionToggled() ) );

	mw->DebugOutput->hide();
	mw->DebugOutput->resize ( QSize ( 0, 0 ) );

	resize( QSize ( 100, 100 ) );

	//mw->CheckUserPass->setText(i18n("save user password"));
	//mw->CheckGroupPass->setText(i18n("save group password"));

	//	mw->SessionCombo->setEnabled( false );
	//mw->SaveSessionButton->setEnabled( false );
	//mw->SessionCombo->insertItem( "   " ); // important: 3 spaces

	/*
	mw->CheckGroupPass->setEnabled ( FALSE );
	mw->CheckUserPass->setEnabled( FALSE );
	mw->GroupPasswordEdit->setEnabled( FALSE );
	mw->IDLineEdit->setEnabled( FALSE );
	mw->PasswordEdit->setEnabled( FALSE );
	mw->SessionCombo->setEnabled( FALSE );
	mw->UsernameLineEdit->setEnabled( FALSE );
	mw->gatewayLineEdit->setEnabled( FALSE );
	mw->SaveSessionButton->setEnabled( FALSE );
	mw->DeleteSessionButton->setEnabled( FALSE );
	*/
	mw->SessionCombo->setEnabled( FALSE );
	//	mw->AdvancedButton->setEnabled( false );
}

void KVpnc::initStatusbar()
{
	statusBar() ->insertItem( i18n( "Ready." ), 1, 100 );
	statusBar() ->setItemAlignment( 1, AlignLeft | AlignVCenter );
	statusBar() ->insertItem( i18n( "Disconnected" ), 2, 0, true );
}

void KVpnc::initDockWindow()
{
	this->kickerMenu = new KVpncKicker( this, "kickerIcon" );
	this->kickerMenu->menu() ->insertSeparator();
	this->connectAction->plug( this->kickerMenu->menu() );
	this->disconnectAction->plug( this->kickerMenu->menu() );

	this->kickerMenu->menu() ->insertSeparator();
	this->preferencesAction->plug( this->kickerMenu->menu() );
}

void KVpnc::saveOptions()
{
	// delete configuration file
	KStandardDirs * dirs = KGlobal::dirs();
	QString filePath = dirs->findResource ( "config", "kvpncrc" );
	QFile::remove
		( filePath );
	this->config->setGroup( "General Options" );

	this->config->writeEntry( "Minimize after connect", minimizeAfterConnect );
	this->config->writeEntry( "Show debugconsole", showDebugConsole );
	this->config->writeEntry( "Vpnc debuglevel", VpncDebugLevel );
	this->config->writeEntry( "Path to vpnc-bin", pathToVpnc );
	this->config->writeEntry( "Path to vpnc-connect", pathToVpncConnect );
	this->config->writeEntry( "Path to vpnc-disconnect", pathToVpncDisconnect );
	this->config->writeEntry( "Show Statusbar", showStatusbarAction->isChecked() );
	this->config->writeEntry( "Try connect timeout", tryConnectTimeout );
	this->config->writeEntry( "Last Profile", lastProfile );
	this->config->writeEntry( "Write log file", writeLogFile );

	toolBar( "mainToolBar" ) ->saveSettings( this->config, "ToolBar Settings" );

	/* = user data = */
	/* old style: one profile: will be removed in future */
	/*
	this->config->setGroup( "Client Options" );
	this->config->writeEntry( "VPN Gateway", this->mw->gatewayLineEdit->text() );
	this->config->writeEntry( "VPN ID", this->mw->IDLineEdit->text() );
	this->config->writeEntry( "Username", this->mw->UsernameLineEdit->text() );
	if ( this->mw->CheckUserPass->isChecked() ) {
		this->config->writeEntry( "Password", this->mw->PasswordEdit->text() );
	}
	if ( this->mw->CheckGroupPass->isChecked() ) {
		this->config->writeEntry( "Group password", this->mw->GroupPasswordEdit->text() );
	}
	this->config->writeEntry( "Save user password", this->mw->CheckUserPass->isChecked() );
	this->config->writeEntry( "Save group password", this->mw->CheckGroupPass->isChecked() );
	*/
	/* new style: multiple profiles: not implemented */
	VpnAccountData *it;
	if ( !AccountList.isEmpty() )
	{
		for ( it = AccountList.first(); it; it = AccountList.next() )
		{
			if ( !it->getName().isEmpty() )
			{
				QString name = it->getName();
				QString ProfileName = "Profile_";
				ProfileName += name;
				this->config->setGroup( ProfileName );
				this->config->writeEntry( "VPN Gateway", it->getGateway() );
				this->config->writeEntry( "VPN ID", it->getID() );
				this->config->writeEntry( "Username", it->getUserName() );
				if ( it->getSaveUserPassword() )
				{
					this->config->writeEntry( "User password", it->getUserPassword() );
				}
				if ( it->getSaveGroupPassword() )
				{
					this->config->writeEntry( "Group password", it->getGroupPassword() );
				}
				this->config->writeEntry( "Save user password", it->getSaveUserPassword() );
				this->config->writeEntry( "Save group password", it->getSaveGroupPassword() );

				this->config->writeEntry( "Use advanced settings", it->getUseAdvancedSettings() );
				this->config->writeEntry( "Application version", it->getApplicationVersion() );
				this->config->writeEntry( "Local port", it->getLocalPort() );
				this->config->writeEntry( "NT domain name", it->getNtDomainName() );
				this->config->writeEntry( "Perfect forward secrety", it->getPerfectForwardSecrety() );
				this->config->writeEntry( "IKE group", it->getIkeGroup() );
				this->config->writeEntry( "Use application version", it->getUseApplicationVersion() );
				this->config->writeEntry( "Use global IPSec secret", it->getUseGlobalIpsecSecret() );
				this->config->writeEntry( "Use IKE group", it->getUseIkeGroup() );
				this->config->writeEntry( "Use local port", it->getUseLocalPort() );
				this->config->writeEntry( "Use NT domain name", it->getUseNtDomainName() );
				this->config->writeEntry( "Use single DES", it->getUseSingleDes() );
				this->config->writeEntry( "Use perfect Forward Secrety", it->getUsePerfectForwardSecrety() );

			}
		}
	}

	this->config->sync();
}

void KVpnc::loadOptions()
{
	toolBar( "mainToolBar" ) ->applySettings( this->config, "ToolBar Settings" );
	showToolbarAction->setChecked( !toolBar( "mainToolBar" ) ->isHidden() );

	this->config->setGroup( "General Options" );
	minimizeAfterConnect = this->config->readBoolEntry( "Minimize after connect", true );
	showDebugConsole = this->config->readBoolEntry( "Show debugconsole", false );
	VpncDebugLevel = this->config->readNumEntry( "Vpnc debuglevel", 0 );
	pathToVpnc = this->config->readEntry( "Path to vpnc-bin", "/usr/sbin/vpnc" );
	pathToVpncConnect = this->config->readEntry( "Path to vpnc-connect", "/usr/sbin/vpnc-connect" );
	pathToVpncDisconnect = this->config->readEntry( "Path to vpnc-disconnect", "/usr/sbin/vpnc-disconnect" );
	tryConnectTimeout = QString( this->config->readEntry( "Try connect timeout", "10" ) ).toInt();
	lastProfile = this->config->readEntry( "Last Profile", "" );
	writeLogFile = this->config->readBoolEntry( "Write log file", true );
	bool bViewStatusbar = this->config->readBoolEntry( "Show Statusbar", true );

	showStatusbarAction->setChecked( bViewStatusbar );
	slotShowStatusbar();
	if ( writeLogFile )
		if ( !logfile.isOpen() )
		{
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc.log";
			//KMessageBox::information( this, logfileName, "info" );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else
			{

				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}


	if ( !showDebugConsole )
	{
		mw->DebugOutput->hide();
		//resize(minimumSizeHint());
		setMinimumSize( minimumSizeHint() );
	}
	else
	{
		mw->DebugOutput->show();
		//resize(minimumSizeHint());
		setMinimumSize( minimumSizeHint() );
	}

	//Failsafe
	if ( this->isHidden() && this->kickerMenu->isHidden() )
	{
		this->show();
	}

	/* = user data = */
	/* old style: one profile: will be removed in future */
	/*
	this->config->setGroup( "Client Options" );
	this->mw->gatewayLineEdit->setText( this->config->readEntry( "VPN Gateway", "" ) );
	this->mw->IDLineEdit->setText( this->config->readEntry( "VPN ID", "" ) );
	this->mw->GroupPasswordEdit->setText( this->config->readEntry( "Group password", "" ) );
	this->mw->UsernameLineEdit->setText( this->config->readEntry( "Username", "" ) );
	this->mw->PasswordEdit->setText( this->config->readEntry( "Password", "" ) );

	this->mw->CheckUserPass->setChecked( this->config->readBoolEntry( "Save user password", false ) );
	this->mw->CheckGroupPass->setChecked( this->config->readBoolEntry( "Save group password", false ) );
	*/
	/* new style: multiple profiles: not implemented */
	/* profiles are stored in a group which is named to the following scheme:
		Profile_XXXXX
		XXXXX is the account name (important: unique!)
	*/
	QStringList configGroups = this->config->groupList();
	bool oldConfigurationFound = false;

	bool sessionFound = false;

	for ( QStringList::Iterator it = configGroups.begin(); it != configGroups.end(); ++it )
	{
		QString name = *it;
		if ( name.left( 7 ) == "Profile" && name.length() > 8 )
		{
			// Profile found

			sessionFound = true;

			QString tmp = i18n( "Profile found: " );
			tmp += name.right( ( name.length() ) - 8 );
			appendLogEntry ( tmp );

			this->config->setGroup( name );
			QString Name = name.right( name.length() - 8 );
			QString Gateway = this->config->readEntry( "VPN Gateway", "" );
			QString ID = this->config->readEntry( "VPN ID", "" ) ;
			QString GroupPassword = this->config->readEntry( "Group password", "" );
			QString Username = this->config->readEntry( "Username", "" );
			QString UserPassword = this->config->readEntry( "User password", "" );
			bool saveUserPassword = this->config->readBoolEntry( "Save user password", false );
			bool saveGroupPassword = this->config->readBoolEntry( "Save group password", false );


			bool useAdvancedSettings = this->config->readBoolEntry( "Use advanced settings", false );
			QString ApplicationVersion = this->config->readEntry( "Application version", "Cisco Systems VPN Client 0.2-rm+zomb.1:Linux" );
			int Port = this->config->readNumEntry( "Local port", 500 );
			QString NtDomainName = this->config->readEntry( "NT domain name", "" );
			QString PerfectForwardSecrecy = this->config->readEntry( "Perfect forward secrety", "server" );
			QString IkeGroup = this->config->readEntry( "IKE group", "dh1" );
			bool useApplicationVersion = this->config->readBoolEntry( "Use application version", false );
			bool useGlobalIpsecSecret = this->config->readBoolEntry( "Use global IPSec secret", false );
			bool useIkeGroup = this->config->readBoolEntry( "Use IKE group", false );
			bool useLocalPort = this->config->readBoolEntry( "Use local port", false );
			bool useNtDomainName = this->config->readBoolEntry( "Use NT domain name", false );
			bool useSingleDes = this->config->readBoolEntry( "Use single DES", false );
			bool usePerfectSecrecy = this->config->readBoolEntry( "Use perfect Forward Secrety", false );


			AccountList.append( new VpnAccountData( Name, Gateway, ID, GroupPassword, Username,
			                                        UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings , usePerfectSecrecy ) );


			//Name, Gateway, ID, GroupPassword, Username, UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings ) );

			if ( !Name.isEmpty() )
				mw->SessionCombo->insertItem( Name );

		}

		if ( sessionFound )
			mw->SessionCombo->setEnabled( TRUE );

		if ( name == "Client Options" )
		{
			// old style configuration found
			this->config->setGroup( "Client Options" );
			QString Name = "Default";
			QString Gateway = this->config->readEntry( "VPN Gateway", "" );
			QString ID = this->config->readEntry( "VPN ID", "" );
			QString GroupPassword = this->config->readEntry( "Group password", "" );
			QString Username = this->config->readEntry( "Username", "" );
			QString UserPassword = this->config->readEntry( "Password", "" );
			bool saveUserPassword = this->config->readBoolEntry( "Save user password", false );
			bool saveGroupPassword = this->config->readBoolEntry( "Save group password", false );


			AccountList.append( new VpnAccountData( Name, Gateway, ID, GroupPassword, Username, UserPassword, saveUserPassword, saveGroupPassword, QString( "" ), 500, QString( "" ), QString( "server" ), QString( "dh1" ), false, false, false, false, false, false, false, false ) );

			mw->SessionCombo->insertItem( Name );
			mw->SessionCombo->setCurrentItem( Name );

			lastProfile = "Default";

			sessionToggled();

			// now mark for deleting old configuration
			oldConfigurationFound = true;
			appendLogEntry ( i18n( "Old configuration found, converted." ) );
		}
	}
	if ( oldConfigurationFound )
	{
		// delete configuration file
		KStandardDirs * dirs = KGlobal::dirs();
		QString filePath = dirs->findResource ( "config", "kvpncrc" );
		QFile::remove
			( filePath );
		appendLogEntry ( i18n( "Old configuration deleted." ) );
	}

	if ( !lastProfile.isEmpty() )
	{
		mw->SessionCombo->setCurrentText( lastProfile );
		sessionToggled();
	}
	else
	{
		if ( !AccountList.isEmpty() )
		{
			mw->SessionCombo->setCurrentItem( 0 );
			sessionToggled();
		}
	}

	mw->groupBox1->minimumSizeHint();

}

bool KVpnc::queryExit()
{
	lastProfile = mw->SessionCombo->currentText();
	this->saveOptions();
	return true;
}

void KVpnc::slotSettings()
{
	this->slotStatusMsg( i18n( "Setup KVpnc..." ), ID_FLASH_MSG );
	this->slotStatusMsg( i18n( "Ready." ), ID_STATUS_MSG );

	PreferencesDialog prefDlg( this, "PreferencesDlg" );
	prefDlg.VpncPathInput->setURL( pathToVpnc );
	prefDlg.VpncConnectPathInput->setURL( pathToVpncConnect );
	prefDlg.VpncDisconnectPathInput->setURL( pathToVpncDisconnect );
	prefDlg.checkMinimizeAfterConnectCheck->setChecked( minimizeAfterConnect );
	prefDlg.showDebugCheck->setChecked( showDebugConsole );
	prefDlg.WriteLogCheck->setChecked( writeLogFile );
	prefDlg.spinVpncDebugLevel->setValue ( VpncDebugLevel );
	prefDlg.spinTryConnectTimeout->setValue( tryConnectTimeout );
	prefDlg.buttonHelp->hide();

	/*
	appendLogEntry ("vpn-connect: "+mw->pathToVpnc);
	appendLogEntry ("vpn-connect: "+mw->pathToVpncConnect);
	appendLogEntry ("vpn-disconnect: "+mw->pathToVpncDisconnect);
	appendLogEntry ("minimize after connect: "+mw->minimizeAfterConnect);
	*/
	prefDlg.exec();

	pathToVpnc = prefDlg.VpncPathInput->url();
	pathToVpncConnect = prefDlg.VpncConnectPathInput->url();
	pathToVpncDisconnect = prefDlg.VpncDisconnectPathInput->url();
	minimizeAfterConnect = prefDlg.checkMinimizeAfterConnectCheck->isChecked();
	showDebugConsole = prefDlg.showDebugCheck->isChecked();
	if ( showDebugConsole )
	{
		mw->DebugOutput->show();
		//resize( minimumSizeHint() );
	}
	else
	{
		mw->DebugOutput->hide();
		/*
		mw->groupBox1->minimumSizeHint();
		mw->DebugOutput->resize( QSize( 0, 0 ) );
		resize( minimumSizeHint() );
		*/
	}
	VpncDebugLevel = prefDlg.spinVpncDebugLevel->value();
	tryConnectTimeout = prefDlg.spinTryConnectTimeout->value();

	if ( writeLogFile )
	{
		if ( !logfile.isOpen() )
		{
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc.log";
			//KMessageBox::information( this, "info", logfileName );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else
			{

				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}
	}
	else
	{
		if ( logfile.isOpen() )
		{
			QTextStream s( &logfile );
			s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			logfile.close();
		}
	}
}

void KVpnc::slotShowToolbar()
{
	if ( !showToolbarAction->isChecked() )
	{
		toolBar( "mainToolBar" ) ->hide();
	}
	else
	{
		toolBar( "mainToolBar" ) ->show();
	}
}

void KVpnc::slotShowStatusbar()
{

	if ( !showStatusbarAction->isChecked() )
	{
		statusBar() ->hide();
	}
	else
	{
		statusBar() ->show();
	}

}


void KVpnc::slotKeyBindings()
{
	//	KKeyDialog::configureKeys( actionCollection(), xmlFile() );
}

void KVpnc::slotConfigToolbars()
{
	KEditToolbar dlg( actionCollection() );

	if ( dlg.exec() )
	{
		createGUI();
	}
}

void KVpnc::slotConfigNotifications()
{
	// 	KNotifyDialog::configure( this, "notifyDialog" );
}


void KVpnc::slotStatusMsg( const QString &text, int id )
{
	if ( id != ID_FLASH_MSG )
	{
		statusBar() ->clear();
		statusBar() ->changeItem( text, id );
	}
	else
	{
		statusBar() ->message( text, 1000 );

	}

}

void KVpnc:: quitCalled()
{
	bool reallyQuit = true;

	if ( this->isConnected )
	{
		int res = KMessageBox::questionYesNo ( this, i18n( "You are still conneced. Really quit?" ), i18n( "Quit?" ) );
		if ( res == 3 )
		{
			disconnectClicked();

			if ( !mw->SessionCombo->isEnabled() &&
			        ( !mw->GroupPasswordEdit->text().isEmpty()
			          || !mw->IDLineEdit->text().isEmpty()
			          || !mw->PasswordEdit->text().isEmpty()
			          || !mw->UsernameLineEdit->text().isEmpty()
			          || !mw->gatewayLineEdit->text().isEmpty()
			        ) )
			{
				int res2 = KMessageBox::questionYesNo ( this, i18n( "You have enter data into fields but dont save it as profile.\nIf You dont save it you will loose the data.\nDo You want save it as new profile?" ), i18n( "Save a session?" ) );
				if ( res2 == 3 )
				{
					if ( hasEmptyFields() )
					{
						int res = KMessageBox::questionYesNo ( this, i18n( "Some fields are blank. This is not allowed.\nYou will lose your data.\nCancel for enterring data?" ), i18n( "Continue?" ) );
						if ( res == 3 )
						{
							reallyQuit = false;
						}
					}
					else
					{
						autoclearFields = false;
						newSessionClicked();
						saveSessionClicked();
					}
				}
			}

			if ( writeLogFile )
			{
				if ( logfile.isOpen() )
				{
					QTextStream s( &logfile );
					s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
					logfile.flush();
					logfile.close();
				}
			}

			kapp->quit();
		}
		//KMessageBox::sorry ( this, QString().setNum(res), "");
	}
	else
	{

		if ( !mw->SessionCombo->isEnabled() &&
		        ( !mw->GroupPasswordEdit->text().isEmpty()
		          || !mw->IDLineEdit->text().isEmpty()
		          || !mw->PasswordEdit->text().isEmpty()
		          || !mw->UsernameLineEdit->text().isEmpty()
		          || !mw->gatewayLineEdit->text().isEmpty()
		        ) )
		{
			int res2 = KMessageBox::questionYesNo ( this, i18n( "You have enter data into fields but dont save it as profile.\nIf You dont save it you will loose the data.\nDo You want save it as new profile?" ), i18n( "Save a session?" ) );
			if ( res2 == 3 )
			{
				if ( hasEmptyFields() )
				{
					int res = KMessageBox::questionYesNo ( this, i18n( "Some fields are blank. This is not allowed.\nYou will lose your data.\nCancel for enterring data?" ), i18n( "Continue?" ) );
					if ( res == 3 )
					{
						reallyQuit = false;
					}
				}
				else
				{
					autoclearFields = false;
					newSessionClicked();
					saveSessionClicked();
				}
			}
		}



		if ( reallyQuit )
		{
			if ( writeLogFile )
			{
				if ( logfile.isOpen() )
				{
					QTextStream s( &logfile );
					s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
					logfile.flush();
					logfile.close();
				}
			}

			kapp->quit();

		}
	}

}

void KVpnc::closeEvent( QCloseEvent* e )
{
	bool reallyQuit = true;
	if ( this->isConnected )
	{
		int res = KMessageBox::questionYesNo ( this, i18n( "You are still conneced. Really quit?" ), i18n( "Quit?" ) );
		if ( res == 3 )
		{
			disconnectClicked();

			if ( !mw->SessionCombo->isEnabled() &&
			        ( !mw->GroupPasswordEdit->text().isEmpty()
			          || !mw->IDLineEdit->text().isEmpty()
			          || !mw->PasswordEdit->text().isEmpty()
			          || !mw->UsernameLineEdit->text().isEmpty()
			          || !mw->gatewayLineEdit->text().isEmpty()
			        ) )
			{
				int res2 = KMessageBox::questionYesNo ( this, i18n( "You have enter data into fields but dont save it as profile.\nIf You dont save it you will loose the data.\nDo You want save it as new profile?" ), i18n( "Save a session?" ) );
				if ( res2 == 3 )
				{
					if ( hasEmptyFields() )
					{
						int res5 = KMessageBox::questionYesNo ( this, i18n( "Some fields are blank. This is not allowed.\nYou will lose your data.\nCancel for enterring data?" ), i18n( "Continue?" ) );
						if ( res5 == 3 )
						{
							e = 0L;
							reallyQuit = false;
						}
					}
					else
					{
						autoclearFields = false;
						newSessionClicked();
						saveSessionClicked();
					}
				}
			}

			if ( reallyQuit )
			{

				saveOptions();

				if ( writeLogFile )
				{
					if ( logfile.isOpen() )
					{
						QTextStream s( &logfile );
						s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
						logfile.close();
					}
				}
				KMainWindow::closeEvent( e );
			}

		}
		else
		{
			e = 0L;
		}

	}
	else
	{
		if ( !mw->SessionCombo->isEnabled() &&
		        ( !mw->GroupPasswordEdit->text().isEmpty()
		          || !mw->IDLineEdit->text().isEmpty()
		          || !mw->PasswordEdit->text().isEmpty()
		          || !mw->UsernameLineEdit->text().isEmpty()
		          || !mw->gatewayLineEdit->text().isEmpty()
		        ) )
		{
			int res3 = KMessageBox::questionYesNo ( this, i18n( "You have enter data into fields but dont save it as profile.\nIf You dont save it you will loose the data.\nDo You want save it as new profile?" ), i18n( "Save a session?" ) );
			if ( res3 == 3 )
			{
				if ( hasEmptyFields() )
				{
					int res = KMessageBox::questionYesNo ( this, i18n( "Some fields are blank. This is not allowed.\nYou will lose your data.\nCancel for enterring data?" ), i18n( "Continue?" ) );
					if ( res == 3 )
					{
						e = 0L;
						reallyQuit = false;
					}
				}
				else
				{
					autoclearFields = false;
					newSessionClicked();
					saveSessionClicked();
				}
			}
		}

		if ( reallyQuit )
		{
			saveOptions();
			if ( writeLogFile )
			{
				if ( logfile.isOpen() )
				{
					s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
					logfile.flush();
					logfile.close();
				}
			}
			KMainWindow::closeEvent( e );
		}

	}

}



void KVpnc::connectClicked()
{
	/*
		QString connectString = "/usr/sbin/vpnc-connect ";
		connectString += " --gateway ";
		connectString += mw->gatewayLineEdit->text();
		connectString += " --id ";
		connectString += mw->IDLineEdit->text();
		connectString += " --username ";
		connectString += mw->UsernameLineEdit->text();
		connectString += "\n";
		//	appendLogEntry ( connectString );
	*/
	bool ok = true; // if ok we will run process

	// looking for vpnc
	if ( pathToVpnc.isEmpty() )
	{
		QFile vpnc( "/usr/sbin/vpnc" );
		if ( vpnc.exists() )
			pathToVpnc = "/usr/sbin/vpnc";
		else
		{
			vpnc.setName( "/usr/local/sbin/vpnc" );
			if ( vpnc.exists() )
				pathToVpnc = "/usr/local/sbin/vpnc";
			else
			{
				vpnc.setName( "/usr/bin/vpnc" );
				if ( vpnc.exists() )
					pathToVpnc = "/usr/bin/vpnc";
				else
				{
					KMessageBox::error( this, i18n( "Unable to find vpnc!" ) );
					ok = false;
				}
			}
		}
	}
	// looking for vpnc-connect
	if ( pathToVpncConnect.isEmpty() )
	{
		QFile vpnc_connect( "/usr/sbin/vpnc-connect" );
		if ( vpnc_connect.exists() )
			pathToVpncConnect = "/usr/sbin/vpnc-connect";
		else
		{
			vpnc_connect.setName( "/usr/local/sbin/vpnc-connect" );
			if ( vpnc_connect.exists() )
				pathToVpncConnect = "/usr/local/sbin/vpnc-connect";
			else
			{
				vpnc_connect.setName( "/usr/bin/vpnc-connect" );
				if ( vpnc_connect.exists() )
					pathToVpncConnect = "/usr/bin/vpnc-connect";
				else
				{
					KMessageBox::error( this, i18n( "Unable to find vpnc-connect!" ) );
					ok = false;
				}
			}
		}
	}

	// looking for vpnc-disconnect
	if ( pathToVpncDisconnect.isEmpty() )
	{
		QFile vpnc_disconnect( "/usr/sbin/vpnc-disconnect" );
		if ( vpnc_disconnect.exists() )
			pathToVpncDisconnect = "/usr/sbin/vpnc-disconnect";
		else
		{
			vpnc_disconnect.setName( "/usr/local/sbin/vpnc-disconnect" );
			if ( vpnc_disconnect.exists() )
				pathToVpncConnect = "/usr/local/sbin/vpnc-disconnect";
			else
			{
				vpnc_disconnect.setName( "/usr/bin/vpnc-disconnect" );
				if ( vpnc_disconnect.exists() )
					pathToVpncDisconnect = "/usr/bin/vpnc-disconnect";
				else
				{
					KMessageBox::error( this, i18n( "Unable to find vpnc-disconnect!" ) );
					ok = false;
				}
			}
		}
	}

	// looking for empty fields
	if ( hasEmptyFields() )
	{
		KMessageBox::sorry( this, i18n( "All fields must be filled in!" ) );
		ok = false;
	}

	// here we go
	if ( ok )
	{
		ConnectProcess = new QProcess ( this );

		ConnectProcess->addArgument( pathToVpncConnect );
		//	ConnectProcess->addArgument("--no-detach");

		if ( VpncDebugLevel > 0 )
		{
			appendLogEntry ( i18n( "Setting vpnc debug level %1." ).arg( QString().setNum( VpncDebugLevel ) ) );
			ConnectProcess->addArgument( "--debug" );
			ConnectProcess->addArgument( QString().setNum( VpncDebugLevel ) );
		}
		ConnectProcess->addArgument( "--gateway" );
		ConnectProcess->addArgument( mw->gatewayLineEdit->text() );
		ConnectProcess->addArgument( "--id" );
		ConnectProcess->addArgument( mw->IDLineEdit->text() );
		ConnectProcess->addArgument( "--username" );
		ConnectProcess->addArgument( mw->UsernameLineEdit->text() );

		VpnAccountData * it;
		VpnAccountData *currentAccount = 0;
		for ( it = AccountList.first(); it; it = AccountList.next() )
		{
			if ( it->getName() == mw->SessionCombo->currentText() )
			{
				currentAccount = it;
				break;
			}
		}

		if ( currentAccount != 0 )
		{

			if ( currentAccount->getUseAdvancedSettings() )
			{

				if ( currentAccount->getUseApplicationVersion() )
				{
					ConnectProcess->addArgument( "--application-version" );
					ConnectProcess->addArgument( currentAccount->getApplicationVersion() );
					appendLogEntry ( i18n( "Using userdefinied application version \"%1\"." ).arg( currentAccount->getApplicationVersion() ) );
				}

				if ( currentAccount->getUseLocalPort() )
				{
					ConnectProcess->addArgument( "--local-port" );
					ConnectProcess->addArgument( QString().setNum( currentAccount->getLocalPort() ) );
					appendLogEntry ( i18n( "Using userdefinied alocal port \"%1\"." ).arg( QString().setNum( currentAccount->getLocalPort() ) ) );
				}

				if ( currentAccount->getUseNtDomainName() )
				{
					ConnectProcess->addArgument( "--domain" );
					ConnectProcess->addArgument( currentAccount->getNtDomainName() );
					appendLogEntry ( i18n( "Using (NT) domain name \"%1\"." ).arg( currentAccount->getNtDomainName() ) );
				}

				if ( currentAccount->getUsePerfectForwardSecrety() )
				{
					ConnectProcess->addArgument( "--pfs" );
					ConnectProcess->addArgument( currentAccount->getPerfectForwardSecrety() );
					appendLogEntry ( i18n( "Using userdefinied perfect forward secrety \"%1\"." ).arg( currentAccount->getPerfectForwardSecrety() ) );
				}

				if ( currentAccount->getUseIkeGroup() )
				{
					ConnectProcess->addArgument( "--dh" );
					ConnectProcess->addArgument( currentAccount->getIkeGroup() );
					appendLogEntry ( i18n( "Using userdefinied IKE group \"%1\"." ).arg( currentAccount->getIkeGroup() ) );
				}

				if ( currentAccount->getUseSingleDes() )
				{
					ConnectProcess->addArgument( "--enable-1des" );
					appendLogEntry ( i18n( "Using single DES." ) );
				}

			}
		}

		connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_connect() ) );
		connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_connect() ) );

		appendLogEntry ( i18n( "Trying to connect to server \"%1\" with user \"%2\" and IPSec ID \"%3\"...\n" ).arg( mw->gatewayLineEdit->text().arg(mw->UsernameLineEdit->text()).arg(mw->IDLineEdit->text() ) ));
		slotStatusMsg( i18n( "Connecting..." ), ID_CONNECTION_MSG );

		if ( !ConnectProcess->start() )
		{
			KMessageBox::sorry( this, i18n( "Unable to start process (vpnc-connect)!" ) );
		}
		else
		{
			appendLogEntry ( i18n( "vpnc-connect started.\n" ) );

			connect( &timer, SIGNAL( timeout() ), this, SLOT( slotConnectTimedOut() ) );
			timer.start( tryConnectTimeout * 1000, FALSE );
			connectionTime.start();

			mw->buttonConnect->setEnabled( false );

			QString GrpPassStr = mw->GroupPasswordEdit->text();
			GrpPassStr += "\n";
			ConnectProcess->writeToStdin( GrpPassStr );

			QString passStr = mw->PasswordEdit->text();
			passStr += "\n";
			ConnectProcess->writeToStdin( passStr );

		}
	}
}


void KVpnc::disconnectClicked()
{
	//if ( isConnected )
	{

		bool ok = true; // if ok we will run process

		if ( pathToVpncConnect.isEmpty() )
		{
			QFile vpnc_connect( "/usr/sbin/vpnc-disconnect" );
			if ( vpnc_connect.exists() )
				pathToVpncConnect = "/usr/sbin/vpnc-disconnect";
			else
			{
				vpnc_connect.setName( "/usr/local/sbin/vpnc-disconnect" );
				if ( vpnc_connect.exists() )
					pathToVpncConnect = "/usr/local/sbin/vpnc-disconnect";
				else
				{
					KMessageBox::sorry( this, i18n( "Unable to find vpnc-disconnect!" ) );
					ok = false;
				}
			}
		}

		// here we go
		if ( ok )
		{

			DisconnectProcess = new QProcess ();
			DisconnectProcess->addArgument( pathToVpncDisconnect );

			if ( VpncDebugLevel > 0 )
			{
				appendLogEntry ( i18n( "Setting vpnc debug level %1." ).arg( QString().setNum( VpncDebugLevel ) ) );
				ConnectProcess->addArgument( "--debug" );
				ConnectProcess->addArgument( QString().setNum( VpncDebugLevel ) );
			}

			connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_disconnect() ) );
			connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_disconnect() ) );

			if ( !DisconnectProcess->start() )
			{
				KMessageBox::sorry( this, i18n( "Unable to start process (vpnc-disconnect)!" ) );
			}
			else
			{
				durationTimer.stop();
				isConnected = false;

				setGuiStatus ( disconnected );

				appendLogEntry ( i18n( "Successful disconnected." ) );

				// tmp
				QString hour_string = QString().sprintf( "%2d", hour );
				QString min_string = QString().sprintf( "%2d", min );
				QString sec_string = QString().sprintf( "%2d", sec );


				QString durationString ( i18n( "%1:%2:%3" ).arg( hour_string ).arg( min_string ).arg( sec_string ) );
				appendLogEntry ( i18n( "Connection duration was %1 hours, %2 minutes, %3 seconds" ).arg( hour_string ).arg( min_string ).arg( sec_string ) );
				connectCounter = 0;
			}
		}
		else
			appendLogEntry ( i18n( "Not connected.\n" ) );
	}
}

void KVpnc::helpClicked()
{
	HelpDialog d1;
	d1.exec();
	slotStatusMsg ( i18n( "Help dialog called." ), ID_FLASH_MSG );
}

void KVpnc::readFromStderr_connect()
{
	//DebugOutput->append( "readFromStderr_connect!\n" );
	if ( ConnectProcess->canReadLineStderr() )
	{

		ProcessMsg_connect = ConnectProcess->readLineStderr();

		bool abort = false;

		//ProcessMsg_connect = QString(ConnectProcess->readStdout());
		if ( this->VpncDebugLevel > 0 )
			this->appendLogEntry ( QString ( "raw stderr: " + ProcessMsg_connect ) );

		// auth failed
		if ( ProcessMsg_connect.find( "AUTHENTICATION_FAILED", 0, FALSE ) > 1 )
		{
			KMessageBox::error( this, i18n( "Authentication failed!" ) );
			abort = true;
		}

		if ( ProcessMsg_connect.find( "authentication unsuccessful", 0, FALSE ) > 1 )
		{
			KMessageBox::error( this, i18n( "Authentication failed!" ) );
			abort = true;
		}

		// no answer
		if ( ProcessMsg_connect.find( "no response from target", 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "No Response from VPN server" ) );
			abort = true;
		}

		// initalizng tunnel interface failed
		if ( ProcessMsg_connect.find( "can't initialise tunnel interface", 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Tunnel interface can't be initalized" ) );
			abort = true;
		}

		// host unknown
		if ( ProcessMsg_connect.find( "unknown host" , 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Host unknown" ) );
			abort = true;
		}

		//socket creation failed
		if ( ProcessMsg_connect.find( "making socket", 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Socket creation failed" ) );
			abort = true;
		}


		// binding to port failed
		if ( ProcessMsg_connect.find( "binding to port", 0 , FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Port binding failed" ) );
			abort = true;
		}


		if ( ProcessMsg_connect.find( "Network is unreachable", 0 , FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "No network reachable" ) );
			terminateConnection ( *ConnectProcess );
		}

		if ( ProcessMsg_connect.find( "vpnc found running", 0 , FALSE ) > -1 )
		{
			//KMessageBox::sorry( this, i18n( "vpnc is already running, kill it manually" ) );
			//terminateConnection ( *ConnectProcess );
			appendLogEntry ( i18n( "Killing old vpnc...\n" ) );
			disconnectClicked();
			appendLogEntry ( i18n( "Trying again...\n" ) );
			connectClicked();
		}

		if ( abort )
		{
			terminateConnection ( *ConnectProcess );
		}

		if ( !ProcessMsg_connect.isEmpty() )
			appendLogEntry ( ProcessMsg_connect + "\n" );
	}
}

void KVpnc::readFromStdout_connect()
{
	//DebugOutput->append( "readFromStdout_connect!\n" );

	if ( ConnectProcess->canReadLineStdout() )
	{
		ProcessMsg_connect = ConnectProcess->readLineStdout();

		//ProcessMsg_connect = QString(ConnectProcess->readStdout());

		if ( this->VpncDebugLevel > 0 )
			this->appendLogEntry ( QString ( "raw stdout: " + ProcessMsg_connect ) );

		// success
		// background -> normal but no check if running :(
		// foreground -> stays in foreground and we can test if running :)
		if ( ProcessMsg_connect.find( "started in background", 0, FALSE ) > 1 ||
		        ProcessMsg_connect.find( "started in foreground", 0, FALSE ) > 1
		   )
		{

			//connectionTime.restart();
			//uint duration = connectionTime.elapsed();

			sec = hour = min = 0;

			isConnected = true;
			timer.stop();
			connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
			durationTimer.start( 1000, FALSE );
			setGuiStatus ( connected );
		}

		if ( ProcessMsg_connect.find( "vpnc found running", 0 , FALSE ) > -1 )
		{
			//KMessageBox::sorry( this, i18n( "vpnc is already running, kill it manually" ) );
			//terminateConnection ( *ConnectProcess );
			appendLogEntry ( i18n( "Killing old vpnc...\n" ) );
			disconnectClicked();
			appendLogEntry ( i18n( "Trying again...\n" ) );
			connectClicked();
		}

		if ( !ProcessMsg_connect.isEmpty() )
			appendLogEntry ( ProcessMsg_connect + "\n" );
		slotStatusMsg ( i18n( "Ready." ), ID_STATUS_MSG );
	}
}

void KVpnc::readFromStderr_disconnect()
{

	ProcessMsg_disconnect = DisconnectProcess->readLineStderr();
	if ( !ProcessMsg_disconnect.isEmpty() )
		appendLogEntry ( ProcessMsg_disconnect + "\n" );
}

void KVpnc::readFromStdout_disconnect()
{
	ProcessMsg_disconnect = DisconnectProcess->readLineStdout();

	if ( !ProcessMsg_disconnect.isEmpty() )
		appendLogEntry ( ProcessMsg_disconnect + "\n" );
}


void KVpnc::terminateConnection ( QProcess & proc )
{
	if ( proc.isRunning() )
		proc.kill();
	timer.stop();
	durationTimer.stop();
	isConnected = false;
	connectCounter = 0;
	setGuiStatus ( disconnected );

}

void KVpnc::slotConnectTimedOut()
{
	this->connectCounter++;

	if ( connectCounter > connectSleepRetry )
	{
		// it dont get it connected so we kill it :)
		appendLogEntry ( i18n( "Timeout! Kill connect process!" ) );
		this->ConnectProcess->kill();
		timer.stop();
		isConnected = false;
		mw->buttonConnect->setEnabled( true );
		mw->buttonDisconnect->setEnabled( false );
		this->connectAction->setEnabled( true );
		this->disconnectAction->setEnabled( false );
		//parent->statusBar() ->insertItem( i18n( "Connected" ), 2, 0, true );
		kickerMenu->setStatus( disconnected );
		appendLogEntry ( i18n( "Connection failed (timeout)." ) );
		disconnectClicked();
		slotStatusMsg( i18n( "Connection timed out, vpnc killed." ), ID_FLASH_MSG );
		slotStatusMsg( i18n( "Disconnected" ), ID_CONNECTION_MSG );
		QToolTip::remove
			( kickerMenu );
		QToolTip::add
			( kickerMenu, i18n( "Disconnected" ) );
		connectCounter = 0;
	}
	else
	{
		appendLogEntry ( i18n( "Waiting %1 seconds for connect..." ).arg( connectSleepDelay ) );
		slotStatusMsg( i18n( "Waiting for connect..." ), ID_FLASH_MSG );
	}
}

void KVpnc::clearFields()
{
	this->mw->gatewayLineEdit->clear();
	this->mw->IDLineEdit->clear();
	this->mw->GroupPasswordEdit->clear();
	this->mw->UsernameLineEdit->clear();
	this->mw->PasswordEdit->clear();
	this->mw->CheckUserPass->setChecked( false );
	this->mw->CheckGroupPass->setChecked( false );
}

void KVpnc::newSessionClicked()
{


	//	if ( autoclearFields )
	clearFields();

	autoclearFields = true;

	QString Name = "";
	bool nameOk = true;
	cancelClicked = false;
	KLineEditDlg dlg( i18n( "Enter name for new Profile:" ), NULL, 0 );
	connect(&dlg, SIGNAL(cancelClicked()), this, SLOT(slotCancelClicked()));
	while ( (!( !Name.isEmpty() && nameOk )) || cancelClicked )
	{
		nameOk = true;

		dlg.exec();
		if ( !cancelClicked )
		{
			Name = dlg.text();

			VpnAccountData * it;
			for ( it = AccountList.first(); it; it = AccountList.next() )
			{
				if ( it->getName() == Name )
				{
					KMessageBox::error ( this, i18n( "Profile name exists!" ), i18n( "Name exist" ) );
					nameOk = false;
					break;
				}
				if ( Name.find( " " ) > -1 || Name.isEmpty() )
				{
					KMessageBox::error ( this, i18n( "Profile name can't contain spaces!" ), i18n( "Illegal character" ) );
					nameOk = false;
					break;
				}
			}
		}
		else
			break;
	}
	if ( !Name.isEmpty() && !cancelClicked){
		mw->SessionCombo->insertItem( Name );
		mw->SessionCombo->setCurrentItem ( Name );
		mw->SessionCombo->setEnabled( TRUE );

		this->deleteSessionAction->setEnabled ( TRUE );
		this->advancedSessionAction->setEnabled ( TRUE );
		this->saveSessionAction->setEnabled ( TRUE );

		/*
		QString ProfileName = "Profile_";
		ProfileName += Name;
		this->config->setGroup( ProfileName );
		*/
		QString Gateway = this->mw->gatewayLineEdit->text();
		QString ID = this->mw->IDLineEdit->text();
		QString Username = this->mw->UsernameLineEdit->text();
		QString UserPassword = "";
		if ( this->mw->CheckUserPass->isChecked() )
		{
			UserPassword = this->mw->PasswordEdit->text();
		}
		QString GroupPassword = "";
		if ( this->mw->CheckGroupPass->isChecked() )
		{
			GroupPassword = this->mw->GroupPasswordEdit->text();
		}
		bool saveUserPassword = this->mw->CheckUserPass->isChecked();
		bool saveGroupPassword = this->mw->CheckGroupPass->isChecked();

		/* currently static for new profile */
		QString ApplicationVersion = "";
		int Port = 500;
		QString NtDomainName = "";
		QString PerfectForwardSecrecy = "";
		QString IkeGroup = "";
		bool useApplicationVersion = false;
		bool useGlobalIpsecSecret = false;
		bool useIkeGroup = false;
		bool useLocalPort = false;
		bool useNtDomainName = false;
		bool useSingleDes = false;
		bool useAdvancedSettings = false;
		bool usePerfectSecrecy = false;

		AccountList.append( new VpnAccountData( Name, Gateway, ID, GroupPassword, Username, UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings, usePerfectSecrecy ) );

		slotStatusMsg ( i18n( "Profile \"%1\" added." ).arg( Name ), ID_FLASH_MSG );
		appendLogEntry ( i18n( "Profile \"%1\" added." ).arg( Name ) );
	}

}

void KVpnc::saveSessionClicked()
{
	VpnAccountData * it;
	if ( !mw->SessionCombo->currentText().isEmpty() )
	{
		for ( it = AccountList.first(); it; it = AccountList.next() )
		{
			if ( it->getName() == mw->SessionCombo->currentText() )
			{

				bool ok = true;
				// looking for empty fields
				if ( hasEmptyFields() )
				{
					KMessageBox::sorry( this, i18n( "All fields must be filled in!" ) );
					ok = false;
				}
				if ( ok )
				{
					it->setGateway( this->mw->gatewayLineEdit->text() );
					it->setID( this->mw->IDLineEdit->text() );
					it->setUserName( this->mw->UsernameLineEdit->text() );

					if ( this->mw->CheckUserPass->isChecked() )
					{
						it->setUserPassword( this->mw->PasswordEdit->text() );
					}
					if ( this->mw->CheckGroupPass->isChecked() )
					{
						it->setGroupPassword( this->mw->GroupPasswordEdit->text() );
					}
					it->setSaveUserPassword( this->mw->CheckUserPass->isChecked() );
					it->setSaveGroupPassword( this->mw->CheckGroupPass->isChecked() );

					slotStatusMsg ( i18n( "Profile \"%1\" saved." ).arg( mw->SessionCombo->currentText() ), ID_FLASH_MSG );
					appendLogEntry ( i18n( "Profile \"%1\" saved." ).arg( mw->SessionCombo->currentText() ) );

				}

			}
		}
	}
}

void KVpnc::deleteSessionClicked()
{
	VpnAccountData * it;
	QString Name = mw->SessionCombo->currentText();
	if ( !mw->SessionCombo->currentText().isEmpty() )
	{
		for ( it = AccountList.first(); it; it = AccountList.next() )
		{
			if ( it->getName() == Name )
			{
				int result = KMessageBox::questionYesNo ( this, i18n( "Really delete %1?" ).arg( Name ), i18n( "Delete?" ) );
				if ( result )
				{
				mw->SessionCombo->removeItem( mw->SessionCombo->currentItem() );
					AccountList.remove( it );
					if ( !AccountList.isEmpty() )
					{
						AccountList.first();
						mw->SessionCombo->setCurrentItem( 0 );
					}
					else
					{
						clearFields();
						this->deleteSessionAction->setEnabled ( FALSE );
						this->advancedSessionAction->setEnabled ( FALSE );
						this->saveSessionAction->setEnabled ( FALSE );

					}

					slotStatusMsg ( i18n( "Profile \"%1\" deleted." ).arg( Name ), ID_FLASH_MSG );
					appendLogEntry ( i18n( "Profile \"%1\" deleted." ).arg( Name ) );
					saveOptions();
					sessionToggled();
					if ( mw->SessionCombo->currentText().isEmpty() )
						mw->SessionCombo->setEnabled ( FALSE );
				}
			}
		}
	}

}

void KVpnc::sessionToggled()
{
	VpnAccountData * it;
	for ( it = AccountList.first(); it; it = AccountList.next() )
	{
		if ( mw->SessionCombo->currentText().isEmpty() )
		{
			mw->gatewayLineEdit->clear();
			mw->IDLineEdit->clear();
			mw->GroupPasswordEdit->clear();
			mw->UsernameLineEdit->clear() ;
			mw->PasswordEdit->clear();
			mw->CheckUserPass->setText( "" );
			mw->CheckGroupPass->setText( "" );
			this->deleteSessionAction->setEnabled ( FALSE );
			this->advancedSessionAction->setEnabled ( FALSE );
			this->saveSessionAction->setEnabled ( FALSE );
		}
		else
		{
			if ( it->getName() == mw->SessionCombo->currentText() )
			{
				mw->gatewayLineEdit->setText( it->getGateway() );
				mw->IDLineEdit->setText( it->getID() );


				mw->GroupPasswordEdit->setText( it->getGroupPassword() );

				if ( it->getUseGlobalIpsecSecret() )
					mw->GroupPasswordEdit->setEnabled( false );
				else
					mw->GroupPasswordEdit->setEnabled( true );


				mw->UsernameLineEdit->setText( it->getUserName() );
				mw->PasswordEdit->setText( it->getUserPassword() );
				mw->CheckUserPass->setChecked( it->getSaveUserPassword() );
				mw->CheckGroupPass->setChecked( it->getSaveGroupPassword() );

				if ( !mw->SessionCombo->currentText().isEmpty() )
				{
					this->deleteSessionAction->setEnabled ( TRUE );
					this->advancedSessionAction->setEnabled ( TRUE );
					this->saveSessionAction->setEnabled ( TRUE );
				}
				else
				{
					this->deleteSessionAction->setEnabled ( FALSE );
					this->advancedSessionAction->setEnabled ( FALSE );
					this->saveSessionAction->setEnabled ( FALSE );
				}

			}

		}
	}
}

void KVpnc::slotDurationEvent()
{
	sec += 1;
	if ( sec > 59 )
	{
		sec = 0;
		min += 1;
	}
	if ( min > 59 )
	{
		min = 0;
		hour += 1;
	}

	// tmp
	QString hour_string = QString().sprintf( "%2d", hour );
	QString min_string = QString().sprintf( "%2d", min );
	QString sec_string = QString().sprintf( "%2d", sec );

	QString durationString ( i18n( "%1:%2:%3" ).arg( hour_string ).arg( min_string ).arg( sec_string ) );
	QToolTip::add
		( kickerMenu, i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID:\"%3\"), duration: %4" ).arg( mw->gatewayLineEdit->text() ).arg( mw->UsernameLineEdit->text() ).arg( mw->IDLineEdit->text() ).arg ( durationString ) );
	slotStatusMsg( i18n( "Connected: %2@%1, %3" ).arg( mw->gatewayLineEdit->text() ).arg( mw->UsernameLineEdit->text() ).arg ( durationString ), ID_CONNECTION_MSG );
	/*
		runningCheckCounter += 1;
		if ( runningCheckCounter == runningCheckTimeout ) {
			if ( !ConnectProcess->isRunning() ) {
				KMessageBox::error( this, i18n( "Connection lost in last %1 seconds!" ).arg( QString().setNum( runningCheckTimeout ) ) );
				appendLogEntry ( i18n( "Connection to %1 lost at %2" ).arg( mw->gatewayLineEdit->text() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ) ) );
				terminateConnection ( *ConnectProcess );
			} else {
				runningCheckCounter = 0;
			}
		}
		*/
}

void KVpnc::appendLogEntry( QString msg )
{

	// write to logfile
	if ( writeLogFile )
	{
		if ( !logfile.isOpen() )
		{
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc.log";
			// 			KMessageBox::information( this, logfileName, "info" );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else
			{

				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}
		// write into debug console
		mw->DebugOutput->append( msg );
	}
}

bool KVpnc::hasEmptyFields()
{
	bool wrong = false;
	// looking for empty fields
	if ( mw->gatewayLineEdit->text() == "" ||
	        mw->IDLineEdit->text() == "" ||
	        mw->GroupPasswordEdit->text() == "" ||
	        mw->UsernameLineEdit->text() == "" ||
	        mw->PasswordEdit->text() == "" )
	{
		wrong = true;
	}
	return wrong;
}

void KVpnc::setGuiStatus( int status )
{
	if ( status == connected )
	{
		setIcon( connectedIcon );

		// tmp
		QString hour_string = QString().sprintf( "%2d", hour );
		QString min_string = QString().sprintf( "%2d", min );
		QString sec_string = QString().sprintf( "%2d", sec );



		QString durationString ( i18n( "%1:%2:%3" ).arg( hour_string ).arg( min_string ).arg( sec_string ) );
		QToolTip::remove
			( kickerMenu );
		QToolTip::add
			( kickerMenu, i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID:\"%3\"), duration: %4" ).arg( mw->gatewayLineEdit->text() ).arg( mw->UsernameLineEdit->text() ).arg( mw->IDLineEdit->text() ).arg ( durationString ) );
		//( kickerMenu, i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID: \"%3\")" ).arg( mw->gatewayLineEdit->text() ).arg( mw->UsernameLineEdit->text() ).arg( mw->IDLineEdit->text() ) );

		slotStatusMsg ( i18n( "Successful connected." ), ID_FLASH_MSG );
		slotStatusMsg( i18n( "Connected: %2@%1, %3" ).arg( mw->gatewayLineEdit->text() ).arg( mw->UsernameLineEdit->text() ).arg ( durationString ), ID_CONNECTION_MSG );
		appendLogEntry ( i18n( "Successful connected to server \"%1\" (user: \"%2\", IPSec ID: \"%3\") at %4" ).arg( mw->gatewayLineEdit->text() ).arg( mw->UsernameLineEdit->text() ).arg( mw->IDLineEdit->text() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ) ) );

		kickerMenu->setStatus( connected );
		mw->buttonConnect->setEnabled( false );
		mw->buttonDisconnect->setEnabled( true );
		this->connectAction->setEnabled( false );
		this->disconnectAction->setEnabled( true );
		if ( minimizeAfterConnect )
			hide();

	}
	else
	{
		setIcon( disconnectedIcon );
		mw->buttonConnect->setEnabled( true );
		mw->buttonDisconnect->setEnabled( false );
		this->connectAction->setEnabled( true );
		this->disconnectAction->setEnabled( false );
		kickerMenu->setStatus( disconnected );
		QToolTip::remove
			( kickerMenu );
		QToolTip::add
			( kickerMenu, i18n( "Disconnected" ) );
		slotStatusMsg( i18n( "Disconnected" ), ID_CONNECTION_MSG );
		slotStatusMsg ( i18n( "Disconnected." ), ID_FLASH_MSG );

	}

}

void KVpnc::advancedSettingsClicked()
{
	VpnAccountData * it = 0L;
	VpnAccountData *currentAccount = 0L;
	AdvancedSettingsDialog d1;

	d1.AdvancedSettingsCheckbox->setChecked ( false );
	d1.UseGlobalIpsecSecretCheckbox->setChecked ( false );
	d1.SingleDesCheckbox->setChecked( false );
	d1.IkeGroupCheckbox->setChecked ( false );
	d1.LocalPortCheckbox->setEnabled ( false );
	d1.PerfectForwardSecurityCheckbox->setChecked( false );
	d1.NtDomainNameCheckbox->setEnabled ( false );
	d1.ApplicationVersionCheckbox->setEnabled ( false );
	d1.UseGlobalIpsecSecretCheckbox->setEnabled ( false );

	if ( mw->SessionCombo->isEnabled() )
	{

		for ( it = AccountList.first(); it; it = AccountList.next() )
		{
			if ( it->getName() == mw->SessionCombo->currentText() )
			{
				currentAccount = it;

				d1.LocalPortSpinbox->setValue( currentAccount->getLocalPort() );
				d1.IkeGroupCombobox->setCurrentText( currentAccount->getIkeGroup() );
				d1.PerfectForwardSecrecyCombobox->setCurrentText( currentAccount->getPerfectForwardSecrety() );
				d1.NtDomainNameLineedit->setText( currentAccount->getNtDomainName() );
				d1.ApplicationVersionLineedit->setText( currentAccount->getApplicationVersion() );

				if ( currentAccount->getUseAdvancedSettings() )
				{
					d1.LocalPortCheckbox->setEnabled ( true );
					d1.IkeGroupCheckbox->setEnabled ( true );
					d1.PerfectForwardSecurityCheckbox->setEnabled ( true );
					d1.SingleDesCheckbox->setEnabled ( true );
					d1.NtDomainNameCheckbox->setEnabled ( true );
					d1.ApplicationVersionCheckbox->setEnabled ( true );
					d1.UseGlobalIpsecSecretCheckbox->setEnabled ( true );
					d1.AdvancedSettingsCheckbox->setChecked ( true );

					if ( currentAccount->getUseSingleDes() )
					{
						d1.SingleDesCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseIkeGroup() )
					{
						d1.IkeGroupCheckbox->setChecked ( true );
					}

					if ( currentAccount->getUseGlobalIpsecSecret() )
					{
						d1.UseGlobalIpsecSecretCheckbox->setChecked ( true );
					}

					if ( currentAccount->getUsePerfectForwardSecrety() )
					{
						d1.PerfectForwardSecrecyCombobox->setEnabled ( true );
						d1.PerfectForwardSecurityCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseNtDomainName() )
					{
						d1.NtDomainNameLineedit->setEnabled ( true );
						d1.NtDomainNameCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseApplicationVersion() )
					{
						d1.ApplicationVersionLineedit->setEnabled ( true );
						d1.ApplicationVersionCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseLocalPort() )
					{
						d1.LocalPortSpinbox->setEnabled ( true );
						d1.LocalPortCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseIkeGroup() )
					{
						d1.IkeGroupCombobox->setEnabled ( true );
					}




				}

				break;
			}
		}

	}


	d1.exec();

	if ( !mw->SessionCombo->currentText()
	        .isEmpty() )
	{
		if ( d1.AdvancedSettingsCheckbox->isChecked() )
		{
			currentAccount->setUseAdvancedSettings( true );
		}
		else
		{
			currentAccount->setUseAdvancedSettings( false );
		}
		if ( d1.NtDomainNameCheckbox->isChecked() )
		{
			currentAccount->setNtDomainName( d1.NtDomainNameLineedit->text() );
			currentAccount->setUseNtDomainName( true );
		}
		else
		{
			currentAccount->setUseNtDomainName( false );
		}

		if ( d1.ApplicationVersionCheckbox->isChecked() )
		{
			currentAccount->setApplicationVersion( d1.ApplicationVersionLineedit->text() );
			currentAccount->setUseApplicationVersion( true );
		}
		else
		{
			currentAccount->setUseApplicationVersion( false );
		}

		if ( d1.IkeGroupCheckbox->isChecked() )
		{
			currentAccount->setIkeGroup( d1.IkeGroupCombobox->currentText() );
			currentAccount->setUseIkeGroup( true );
		}
		else
		{
			currentAccount->setUseIkeGroup( false );
		}

		if ( d1.PerfectForwardSecurityCheckbox->isChecked() )
		{
			currentAccount->setPerfectForwardSecrety( d1.PerfectForwardSecrecyCombobox->currentText() );
			currentAccount->setUsePerfectForwardSecrety( true );
		}
		else
		{
			currentAccount->setUsePerfectForwardSecrety( false );
		}


		if ( d1.SingleDesCheckbox->isChecked() )
		{
			currentAccount->setUseSingleDes( true );
		}
		else
		{
			currentAccount->setUseSingleDes( false );
		}

		if ( d1.UseGlobalIpsecSecretCheckbox->isChecked() )
		{
			currentAccount->setUseGlobalIpsecSecret( true );
		}
		else
		{
			currentAccount->setUseGlobalIpsecSecret( false );
		}


		if ( d1.LocalPortCheckbox->isChecked() )
		{
			currentAccount->setLocalPort( d1.LocalPortSpinbox->value() );
			currentAccount->setUseLocalPort( true );
		}
		else
		{
			currentAccount->setUseLocalPort( false );
		}

	}
	else
	{
		currentAccount->setUseAdvancedSettings( false );
	}
	saveSessionClicked();
	if ( it->getUseGlobalIpsecSecret()
	   )
		this->mw->GroupPasswordEdit->setEnabled( false );
	else
		this->mw->GroupPasswordEdit->setEnabled( true );

}

void KVpnc::slotCancelClicked()
{
	cancelClicked = true;
}

