/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   Contains parts of code from klcc project                              *
*   (Peter Simonsson <psn@linux.se>.                                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef __KVPNC_H__
#define __KVPNC_H__

#include <qwidget.h>
#include <qstrlist.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qfile.h>
#include <qdatetime.h>
#include <qptrlist.h>
#include <qtextstream.h>
#include <qpixmap.h>
#include <kapp.h>
#include <kmainwindow.h>
#include <kaction.h>
#include <kconfig.h>
#include <kprocess.h>
#include "kvpnckicker.h"
#include "preferencesdialog.h"
#include "mainview.h"
#include "vpnaccountdata.h"

// ID for statusbar
#define ID_STATUS_MSG 1
#define ID_CONNECTION_MSG 2
#define ID_FLASH_MSG 3

/**
* This is the main class
* @author Christoph Thielecke
*/
class KVpnc : public KMainWindow {
	Q_OBJECT
public:
	/**
	* Konstruktor
	* @param parent Parent widget
	* @param name Name of the Mainwindow
	*/
	KVpnc( QWidget*parent = 0, const char *name = 0 );

	/**
	* Destruktor
	*/
	~KVpnc();
protected:
	/**
	* init the actions
	*/
	void initAction();

	/**
	* init the main view
	*/
	void initView();

	/**
	* init the statusbar
	*/
	void initStatusbar();

	/**
	* init the kicker system tray icon
	*/
	void initDockWindow();

	/**
	* save configuration
	*/
	void saveOptions();

	/**
	* load configuration
	*/
	void loadOptions();

	/**
	* sets the gui status
	* @param status The status that will be set
	*/
	void setGuiStatus(int status);

	/**
	* terminates the connection
	* @param proc the connect process
	*/
	void terminateConnection ( QProcess& proc );
	/**
	* called when quit requested
	*/
	virtual bool queryExit();

private:
	KVpncKicker* kickerMenu; //< kicker menu
	KAction* connectAction; //< connect action
	KAction* disconnectAction; //< disconnect action
	KAction *newSessionAction;
	KAction *advancedSessionAction;
	KAction *saveSessionAction;
	KAction *deleteSessionAction;
	KAction* selectAction; //< line select action
	KToggleAction* showToolbarAction; //< toggle toolbar on/off
	KToggleAction* showStatusbarAction; //< toggle statusbar on/off
	KAction* preferencesAction; //< show preferences dialog
	KConfig* config; //< configuration data
	MainView* mw; //< Main gui
	int connectCounter; //< counter for connecting
	QTimer timer;
	QTimer durationTimer; //< timer for counting connection time
	int connectSleepDelay; //< time beetween retry
	int connectSleepRetry; //< count of retries
	int tryConnectTimeout; //< timeout in at connecting
	bool cancelClicked;

protected:
	bool minimizeAfterConnect;
	bool showDebugConsole;
	bool isConnected;
	bool writeLogFile;
	bool autoclearFields;
	int VpncDebugLevel;
	int connectionStartTime;
	int runningCheckCounter;
	int runningCheckTimeout;
	int sec, min,hour;
	QString pathToVpnc;
	QString pathToVpncConnect;
	QString pathToVpncDisconnect;
	QString ProcessMsg_connect;
	QString ProcessMsg_disconnect;
	QString lastProfile;
	QString logfileName;
	QProcess *ConnectProcess;
	QProcess *DisconnectProcess;
	QPtrList<VpnAccountData> AccountList;
	QTime connectionTime;
	QFile logfile;
	QTextStream s;
	QPixmap connectedIcon;
	QPixmap disconnectedIcon;

	enum status {disconnected, connected};

public slots:
	/** setup KVpnc */
	void slotSettings();
	/** show / hide toolbar */
	void slotShowToolbar();
	/** show / hide statusbar */
	void slotShowStatusbar();
	/** configure key bindings */
	void slotKeyBindings();
	/** configure toolbars */
	void slotConfigToolbars();

	void slotConfigNotifications();

	/**
	* Append a log entry
	* @param msg The string that will be appended
	*/
	void appendLogEntry(QString msg);

	/**
	* Slot for status message
	* @param text The text that will be set
	* @param id What type its is
	*/
	void slotStatusMsg(const QString &text, int id = ID_STATUS_MSG);

	/**
	* Checks for expty fields in mainwindow
	*/
	bool hasEmptyFields();

	/**
	* clean up the account data fields in mainwindow
	*/
	void clearFields();

public slots:
	void connectClicked();
	void disconnectClicked();
	void saveSessionClicked();
	void newSessionClicked();
	void deleteSessionClicked();
	void sessionToggled();
	void helpClicked();

private slots:
	void closeEvent( QCloseEvent* e );
	void slotCancelClicked();
	void slotConnectTimedOut();
	void slotDurationEvent();
	void readFromStdout_connect();
	void readFromStderr_connect();
	void readFromStdout_disconnect();
	void readFromStderr_disconnect();
	void quitCalled();
	void advancedSettingsClicked();
};

#endif

