/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef VPNACCOUNTDATA_H
#define VPNACCOUNTDATA_H

#include <qstring.h>
#include <qobject.h>

/**
* holds the data for a account
* @author Christoph Thielecke
*/
class VpnAccountData : public QObject {
public:
	/**
	* Konstruktor
	* @param Name The name of the account
	* @param Gateway The hostname/IP of the VPN gateway
	* @param ID The id of the VPN
	* @param GroupPassword The IPSec group secret
	* @param Username The username
	* @param UserPassword The password of the user
	* @param saveUserPassword True if the password of user should be stored
	* @param saveGroupPassword True if the IPSec group secret should be stored
	* @param ApplicationVersion Userdefinied application version string
	* @param Port Userdefinied port where vpnc should listen
	* @param NtDomainName Domain name of NT server for authention (optional)
	* @param PerfectForwardSecrecy What PFS should be used
	* @param IkeGroup Which IKE group should be used
	* @param useApplicationVersion True if userdefinied application version should be sent
	* @param useGlobalIpsecSecret True if the global IPSec secret from /etc/vpnc/default.conf should be used
	* @param useIkeGroup True if userdefinied IKE should be used
	* @param useLocalPort True if userdefinied port should be used
	* @param useNtDomainName True if NT domain name should be used for authention
	* @param useSingleDes True if single DES should be used
	* @param useAdvancedSettings True if some advanced settings should be enabled
	* @param usePerfectSecrecy True if userdefinied PFs should be used
	*/
	VpnAccountData( QString Name, QString Gateway, QString ID,
	                QString GroupPassword, QString Username,
	                QString UserPassword, bool saveUserPassword,
	                bool saveGroupPassword, QString ApplicationVersion, int Port, QString NtDomainName, QString PerfectForwardSecrecy, QString IkeGroup, bool useApplicationVersion, bool useGlobalIpsecSecret, bool useIkeGroup, bool useLocalPort, bool useNtDomainName, bool useSingleDes, bool useAdvancedSettings , bool usePerfectSecrecy);

	/**
	* Destruktor
	*/
	~VpnAccountData();

	QString getName(); //< returns Name
	QString getGateway(); //< returns Gateway
	QString getID(); //< returns ID
	QString getGroupPassword(); //< returns GroupPassword
	QString getUserName(); //< returns UserName
	QString getUserPassword(); //< returns UserPassword
	QString getIkeGroup(); //< returns IkeGroup
	QString getPerfectForwardSecrety(); //< returns PerfectForwardSecrecy
	QString getNtDomainName(); //< returns NtDomainName
	QString getApplicationVersion(); //< returns ApplicationVersion
	bool getSaveUserPassword(); //< returns saveUserPassword
	bool getSaveGroupPassword(); //< returns saveGroupPassword
	bool getUseSingleDes(); //< returns useSingleDes
	bool getUseIkeGroup(); //< returns useIkeGroup
	bool getUsePerfectForwardSecrety(); //< returns usePerfectSecrecy
	bool getUseNtDomainName(); //< returns useNtDomainName
	bool getUseApplicationVersion(); //< returns useApplicationVersion
	bool getUseLocalPort(); //< returns useLocalPort
	bool getUseAdvancedSettings(); //< returns useAdvancedSettings
	bool getUseGlobalIpsecSecret(); //< returns useGlobalIpsecSecret
	int getLocalPort(); //< returns LocalPort


	void setName( QString Name );
	void setGateway( QString Gateway );
	void setID( QString ID );
	void setGroupPassword( QString GroupPassword );
	void setUserName( QString Username );
	void setUserPassword( QString UserPassword );
	void setSaveUserPassword( bool saveUserPassword );
	void setSaveGroupPassword( bool saveGroupPassword );
	void setIkeGroup( QString IkeGroup );
	void setPerfectForwardSecrety( QString PerfectForwardSecrecy );
	void setNtDomainName( QString Name );
	void setApplicationVersion( QString version );
	void setUseSingleDes( bool useSingleDes );
	void setLocalPort( int port );
	void setUseIkeGroup( bool useIkeGroup);
	void setUsePerfectForwardSecrety(bool usePerfectForwardSecrety);
	void setUseNtDomainName(bool useNtDomainName);
	void setUseApplicationVersion(bool useApplicationVersion);
	void setUseLocalPort(bool useLocalPort);
	void setUseAdvancedSettings(bool useAdvancedSettings);
	void setUseGlobalIpsecSecret(bool useGlobalIpsecSecret);
protected:
	QString Name;
	QString Gateway;
	QString ID;
	QString GroupPassword;
	QString Username;
	QString UserPassword;
	QString IkeGroup;
	QString PerfectForwardSecrecy;
	QString NtDomainName;
	QString ApplicationVersion;
	bool saveUserPassword;
	bool saveGroupPassword;
	bool useAdvancedSettings;
	bool useIkeGroup;
	bool usePerfectForwardSecrety;
	bool useNtDomainName;
	bool useApplicationVersion;
	bool useLocalPort;
	bool useSingleDes;
	bool useGlobalIpsecSecret;
	int LocalPort;


};

#endif
